/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __ARTICLE_H__
#define __ARTICLE_H__

#include <time.h>
#include <glib.h>
#include "group.h"
#include "pan-callback.h"
#include "pan-object.h"

#define ARTICLE(a) ((Article *)a)

enum {
	STATE_NONE		= 0,
	STATE_MULTIPART_ALL	= 1<<0,
	STATE_MULTIPART_SOME	= 1<<1,
	STATE_DOWNLOAD_QUEUED	= 1<<2,
	STATE_DECODE_QUEUED	= 1<<3,
	STATE_DECODED		= 1<<4,
	STATE_ERROR		= 1<<5,
	STATE_NEW		= 1<<6,
	STATE_CACHED		= 1<<7,
	STATE_DECODE_FAILED	= 1<<8,
	STATE_KILLFILE		= 1<<9, /* only used in articlelist */
	STATE_WATCHED		= 1<<10 /* only used in articlelist */
};

typedef struct _Article
{
        /* parent class */
	PanObject pan_object;

	guint self_passes_filter : 1;
	guint tree_passes_filter : 1;
	guint dirty : 1;
	gushort unread_children;      /* used in articlelist */

	/* location info -- protected -- this will change.
	   At some point we may need to have group/number pairs
	   for 'article-centric'. */
	Group * group;

	/* multipart info */
	gushort parts;		      /* # of parts in multipart message */
	gushort part;		      /* part #; 0 if no attachments */
	gushort linecount;	      /* # of lines in the body */
	gushort crosspost_qty;	      /* # of groups posted to */
	gushort state;		
	time_t date;                  /* date posted */

	/* basic info */
	gulong number;
	char * subject;
	char * author;
	char * message_id;

	char * filename;              /* maybe NULL. decoded file's name */
	char * references;            /* (read only) maybe NULL. */
	char * reply_to;              /* (read only) maybe NULL. */
	char * followup_to;           /* (read only) maybe NULL. */
	char * newsgroups;            /* (read only) maybe NULL. */

	/* articlelist */
	struct _Article * parent;     /* parent in threaded view */
	GSList * threads;             /* linked list of child threads */
}
Article;


/**
***  PROTECTED
**/

void      article_init                   (void);

void      article_constructor            (Article         * article);

void      article_destructor             (PanObject       * article);

/**
***  PUBLIC LIFE CYCLE
**/

Article*  article_new                    (void);

/**
***  PUBLIC MUTATORS
**/

void      article_set_dirty              (Article         * article,
                                          gboolean          dirty);

void      article_set_references         (Article         * article,
                                          const gchar     * references);

void      article_set_reply_to           (Article         * article,
                                          const gchar     * reply_to);

void      article_set_followup_to        (Article         * article,
                                          const gchar     * followup_to);

void      article_set_newsgroups         (Article         * article,
                                          const gchar     * newsgroups);

void      article_add_flag               (Article         * article,
                                          gushort           flag_to_set);

void      article_remove_flag            (Article         * article,
                                          gushort           flag_to_remove);

void      articles_unref                 (Article        ** articles,
                                          size_t            qty);

/**
***  READ
**/

gboolean  article_is_read      (const Article   * article);

void      article_set_read     (Article         * article,
                                gboolean          read);

void      articles_set_read    (Article        ** articles,
                                int               article_qty,
                                gboolean          read);
                    
/**
***
**/

void articles_delete           (Article        ** articles,
                                int               article_qty,
                                gboolean          sync_database);

/**
***
**/

#define article_flag_on(a, flag) (a && (a->state&flag))


/**
***
**/

/**
 * @param article the article whose thread is to be returned
 * @param GPtrArray a container to hold all the articles found in
 *                  this article's thread.
 */
void article_get_entire_thread   (Article     * article,
                                  GPtrArray   * setme);

void article_get_subthread       (Article     * article,
                                  GPtrArray   * setme);

void article_copy_to_folder      (Article     * article,
                                  Group       * from,
                                  Group       * to);

extern PanCallback* article_deleted;

/**
 * This reparents all of article's children to be children of article's parent,
 * removes article from its parent's child list, and
 * clears out all of article's parent/child references.
 */
void article_isolate (Article*);

gchar* article_get_thread_message_id (const Article*);

#endif /* __ARTICLE_H__ */
